use std::collections::HashMap;

use rocket_dyn_templates::tera::{try_get_value, Error, Value, to_value};

/// If the `value` is not passed, optionally discard all elements where the attribute is null. (include_null -> all, none, only)
pub fn advanced_filter(value: &Value, args: &HashMap<String, Value>) -> Result<Value, Error> {
    let mut arr = try_get_value!("filter", "value", Vec<Value>, value);
    if arr.is_empty() {
        return Ok(arr.into());
    }

    let key = match args.get("attribute") {
        Some(val) => try_get_value!("filter", "attribute", String, val),
        None => return Err(Error::msg("The `filter` filter has to have an `attribute` argument")),
    };

    let null = match args.get("include_null") {
        Some(val) => try_get_value!("filter", "attribute", String, val),
        None => "none".to_string(),
    };

    let against_value = match args.get("value") {
        Some(val) => Some(try_get_value!("filter", "value", String, val)),
        None => None
    };

    if null == "only" && against_value.is_some() {
        return Err(Error::msg("The `filter` filter cannot have both `include_null=only` and `value`"))
    }

    arr = arr
        .into_iter()
        .filter(|v| {
            let tested_value = v.get(key.clone()).unwrap_or(&Value::Null);
            if tested_value.is_null() {
                return match null.as_str() {
                    "all" => true,
                    "none" => false,
                    "only" => true,
                    _ => false,
                }
            } else if null != "only" {
                let val = tested_value.as_str();
                match val {
                    Some(match_v) => {
                        match against_value.clone() {
                            Some(against_v) => match_v == against_v,
                            None => true,
                        }
                    }
                    None => true,
                }
            } else {
                false
            }
        })
        .collect::<Vec<_>>();

    Ok(to_value(arr).unwrap())
}

#[cfg(test)]
mod tests {
    use super::*;
    use serde_json::json;
    use std::collections::HashMap;

    fn call_advanced_filter(data: &Vec<TestClass>, include_null: &str, value: Option<&str>) -> Result<Value, Error> {
        let mut args = HashMap::new();
        args.insert("attribute".to_string(), json!("against_value"));
        args.insert("include_null".to_string(), json!(include_null));
        if let Some(val) = value {
            args.insert("value".to_string(), json!(val));
        }
        advanced_filter(&json!(data), &args)
    }

    #[derive(serde::Serialize, serde::Deserialize, Debug)]
    struct TestClass {
        against_value: Option<String>,
    }

    impl TestClass {
        pub fn new(against_value: Option<String>) -> Self {
            TestClass { against_value }
        }
    }

    fn generate_test_class(first_data: Option<String>, second_data: Option<String>) -> Vec<TestClass> {
        vec![TestClass::new(first_data), TestClass::new(second_data)]
    }

    #[test]
    fn test_advanced_filter() {
        let data = vec![TestClass::new(Some("foo".to_string())), TestClass::new(None)];
        let result = call_advanced_filter(&data, "all", None).unwrap();
        assert_eq!(result, json!(generate_test_class(Some("foo".to_string()), None)));
        let result = call_advanced_filter(&data, "none", None).unwrap();
        assert_eq!(result,  json!(vec![TestClass::new(Some("foo".to_string()))]));
        let result = call_advanced_filter(&data, "only", None).unwrap();
        assert_eq!(result, json!(vec![TestClass::new(None)]));

        // Test filtering strings
        let result = call_advanced_filter(&data, "all", Some("foo")).unwrap();
        assert_eq!(result, json!(vec![TestClass::new(Some("foo".to_string())), TestClass::new(None)]));
        let result = call_advanced_filter(&data, "none", Some("bar")).unwrap();
        let vec: Vec<TestClass> = vec![];
        assert_eq!(result, json!(vec));
        let result = call_advanced_filter(&data, "only", Some("zz")).is_err();
        assert!(result);
    }
}
